#!/usr/bin/env python

"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

import scif
import argparse
import sys
import os


def get_parser():
    parser = argparse.ArgumentParser(description="scientific filesystem tools")

    # Global

    parser.add_argument(
        "--debug",
        dest="debug",
        help="use verbose logging to debug.",
        default=False,
        action="store_true",
    )

    parser.add_argument(
        "--quiet",
        dest="quiet",
        help="suppress print output",
        default=False,
        action="store_true",
    )

    parser.add_argument(
        "--writable",
        "-w",
        dest="writable",
        help="for relevant commands, if writable SCIF is needed",
        default=False,
        action="store_true",
    )

    description = "actions for Scientific Filesystem"
    subparsers = parser.add_subparsers(
        help="scif actions", title="actions", description=description, dest="command"
    )

    version = subparsers.add_parser("version", help="show software version")

    # Shell and Interactive Terminal (it)

    pyshell = subparsers.add_parser(
        "pyshell", help="Interactive python shell to scientific filesystem"
    )

    pyshell.add_argument(
        "recipe", nargs="*", help="recipe file or scientific filesystem base", type=str
    )

    shell = subparsers.add_parser(
        "shell", help="shell to interact with scientific filesystem"
    )

    shell.add_argument(
        "app",
        nargs="?",
        help="app shell to, defaults to SCIF base if not set.",
        type=str,
    )

    preview = subparsers.add_parser("preview", help="preview changes to a filesytem")

    # if the user provides more than one argument here, they are apps
    preview.add_argument(
        "recipe", nargs="*", help="recipe file for the filesystem", type=str
    )

    # Help
    help = subparsers.add_parser("help", help="look at help for an app, if it exists.")

    help.add_argument("app", nargs="*", help="app(s) to print help for", type=str)

    # Install

    install = subparsers.add_parser(
        "install", help="install a recipe on the filesystem"
    )

    install.add_argument(
        "recipe", nargs="*", help="recipe file for the filesystem", type=str
    )

    # Inspect

    inspect = subparsers.add_parser(
        "inspect", help="inspect an attribute for a scif installation"
    )

    inspect.add_argument(
        "attributes",
        nargs="*",
        help="""attribute to inspect (runscript|r),
                                                      (environment|e),
                                                      (labels|l), or (all|a) (default)""",
    )

    # Run

    run = subparsers.add_parser("run", help="entrypoint to run a scientific filesystem")

    run.add_argument(
        "cmd",
        nargs="*",
        help="app and optional arguments to target for the entry",
        type=str,
    )

    # Test

    test = subparsers.add_parser(
        "test", help="entrypoint to test an app in a scientific filesystem"
    )

    test.add_argument(
        "cmd",
        nargs="*",
        help="app and optional arguments to target for the entry",
        type=str,
    )

    # List and dump

    ls = subparsers.add_parser("apps", help="list apps installed")

    ls.add_argument(
        "-l",
        dest="longlist",
        help="show long listing, including paths",
        default=False,
        action="store_true",
    )

    dump = subparsers.add_parser("dump", help="dump recipe")

    # Execute

    execute = subparsers.add_parser(
        "exec", help="execute a command to a scientific filesystem"
    )

    execute.add_argument(
        "cmd",
        nargs="*",
        help="app and command to execute. Eg, exec appname echo $SCIF_APPNAME",
        type=str,
    )

    return parser


def get_subparsers(parser):
    """get_subparser will get a dictionary of subparsers, to help with printing help"""

    actions = [
        action
        for action in parser._actions
        if isinstance(action, argparse._SubParsersAction)
    ]

    subparsers = dict()
    for action in actions:
        # get all subparsers and print help
        for choice, subparser in action.choices.items():
            subparsers[choice] = subparser

    return subparsers


def main():
    """main is the entrypoint to managing or interacting with an scif
    organized filesystem.
    """

    parser = get_parser()
    subparsers = get_subparsers(parser)

    def help(return_code=0):
        """print help, including the software version and active client
        and exit with return code.
        """
        version = scif.__version__

        print("\nScientific Filesystem [v%s]" % (version))
        parser.print_help()
        sys.exit(return_code)

    # If the user didn't provide any arguments, show the full help
    if len(sys.argv) == 1:
        help()
    try:
        args, unknown = parser.parse_known_args()
    except:
        sys.exit(0)

    # If unknown arguments were provided, pass on to cmd to run
    if args.command in ["run", "exec"] and len(unknown) > 0:
        args.cmd += unknown

    # if environment logging variable not set, make silent
    if args.debug is False:
        os.environ["SCIF_MESSAGELEVEL"] = "INFO"
        os.putenv("SCIF_MESSAGELEVEL", "INFO")

    if args.quiet is True:
        os.environ["SCIF_MESSAGELEVEL"] = "QUIET"
        os.putenv("SCIF_MESSAGELEVEL", "QUIET")

    # Show the version and exit
    if args.command == "version":
        print(scif.__version__)
        sys.exit(0)

    # Does the user want a shell?
    if args.command == "apps":
        from .list import main
    if args.command == "dump":
        from .dump import main
    if args.command == "exec":
        from .execute import main
    if args.command == "help":
        from .help import main
    if args.command == "inspect":
        from .inspect import main
    if args.command == "install":
        from .install import main
    if args.command == "preview":
        from .preview import main
    if args.command == "pyshell":
        from .pyshell import main
    if args.command == "run":
        from .run import main
    if args.command == "shell":
        from .shell import main
    if args.command == "test":
        from .test import main

    # Pass on to the correct parser
    return_code = 0
    try:
        main(args=args, parser=parser, subparser=subparsers[args.command])
        sys.exit(return_code)
    except UnboundLocalError:
        return_code = 1

    help(return_code)


if __name__ == "__main__":
    main()



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import tempfile
import os
import pwd

################################################################################
# environment / options
################################################################################


def convert2boolean(arg):
    """
    convert2boolean is used for environmental variables
    that must be returned as boolean
    """
    if not isinstance(arg, bool):
        return arg.lower() in ("yes", "true", "t", "1", "y")
    return arg


def getenv(variable_key, default=None, required=False, silent=True):
    """getenv will attempt to get an environment variable. If the variable
    is not found, None is returned.

    Paramters
    =========
    variable_key: the variable name
    required: exit with error if not found
    silent: Do not print debugging information for variable
    """
    variable = os.environ.get(variable_key, default)
    if variable is None and required:
        bot.exit("Cannot find environment variable %s, exiting." % variable_key)

    if not silent and variable is not None:
        bot.verbose("%s found as %s" % (variable_key, variable))

    return variable


def getenv_namespace(namespace="SCIF", func=None):
    """return all environment variables in a particular namespace, such as for
    SCIF apps using a filter function (func). The function should take first
    a key in the environment, and then the namespace variable.
    If none provided, uses "starts with" equivalent.

     def func(key, namespace):
         return key.startswith(namespace)
    """
    if func is None:
        """does key start with namespace?"""

        def func(key, namespace):
            return key.startswith(namespace)

    env = os.environ.items()
    return [{key: value} for key, value in env if func(key, namespace)]


################################################################################
# Defaults


# Supported Sections
sections = (
    "appenv",
    "applabels",
    "appinstall",
    "appfiles",
    "apphelp",
    "apprun",
    "apptest",
)


################################################################################
# Scientific Filesystem (environment)


#########################
# Global Settings
#########################

SCIF_APPS = getenv_namespace(namespace="SCIF_APP")
SCIF_BASE = getenv("SCIF_BASE", "/scif")
SCIF_DATA = getenv("SCIF_DATA", "%s/data" % SCIF_BASE)
SCIF_APPS = getenv("SCIF_APPS", "%s/apps" % SCIF_BASE)
SCIF_PYSHELL = getenv("SCIF_PYSHELL", "ipython")
SCIF_SHELL = getenv("SCIF_SHELL", "/bin/bash")
SCIF_ENTRYPOINT = getenv("SCIF_ENTRYPOINT", "/bin/bash")
SCIF_ENTRYFOLDER = getenv("SCIF_ENTRYFOLDER")  # default None, is set to /scif
# in functions OR app roots

#########################
# Permissions
#########################

SCIF_APPEND_PATHS = ["PYTHONPATH", "PATH", "LD_LIBRARY_PATH"]
SCIF_ALLOW_APPEND = convert2boolean(getenv("SCIF_ALLOW_APPEND", True))



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import pwd
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    apps = args.recipe

    if len(apps) == 0:
        bot.exit("You must provide a recipe (.scif) file to install.")

    recipe = apps.pop(0)

    if not os.path.exists(recipe):
        bot.exit("Cannot find recipe file %s" % recipe)

    if len(apps) == 0:
        apps = None

    client = ScifRecipe(path=recipe)  # writable is True

    # Preview the entire recipe, or the apps chosen
    client.install(apps)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import sys
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    client = ScifRecipe(writable=False, quiet=True)
    longlist = args.longlist

    result = []
    for app in client.apps():
        config = client.get_appenv(app)

        # Long listing includes a number with path to app
        if longlist is True:
            result.append([app.rjust(10), config["SCIF_APPROOT"]])
        else:
            result.append(app.rjust(10))

    if len(result) > 0:
        if longlist is True:
            header = "[app]              [root]"
            bot.custom(prefix="SCIF", message=header, color="CYAN")
            bot.table(result)
        else:
            print("\n".join(result))



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.defaults import SCIF_BASE


import sys
import os


def parse_input_preferences(recipe, quiet=False):
    """parse a recipe|app, meaning a list of one or more things that could be

    any of the following:

     Returns
     =======
     parsed: a dictionary lookup with keys for app, recipe, and quiet.
    """

    # if coming from inspect, will be string
    if not isinstance(recipe, list):
        recipe = [recipe]

    # First case: recipe and app
    if len(recipe) > 1:
        app = recipe[1]
        recipe = recipe[0]
        quiet = True

    # Second case, no input or recipe|app
    else:
        app = None

        # If no recipe provided, assume loading base

        if len(recipe) == 0:
            recipe = SCIF_BASE

        # Otherwise, we need to figure out if recipe or base
        else:
            recipe = recipe[0]

            # If recipe doesn't exist as a file, assume it's an app
            if not os.path.exists(recipe):
                app = recipe
                recipe = SCIF_BASE

            # If the recipe provided is a file, no app detection
            else:
                quiet = True

    # Return the parsed data structure
    parsed = {"quiet": quiet, "app": app, "recipe": recipe}

    return parsed



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

import errno
import os
import re
import shutil
import stat
import codecs
import json
from scif.logger import bot


################################################################################
## FOLDER OPERATIONS ###########################################################
################################################################################


def mkdir_p(path):
    """mkdir_p attempts to get the same functionality as mkdir -p
    :param path: the path to create.
    """
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            bot.exit("Error creating path %s, exiting." % path)


def which(software):
    """which is a substitute for shutil.which, which is only supported in
    python 3

    Parameters
    ==========
    software: the name of the executable to find

    """
    path = os.getenv("PATH")
    for p in path.split(os.path.pathsep):
        p = os.path.join(p, software)
        if os.path.exists(p) and os.access(p, os.X_OK):
            return p


################################################################################
## FILE OPERATIONS #############################################################
################################################################################


def make_executable(filename):
    """make a file executable by doing the equivalent of chmod+x

    Parameters
    ==========
    filename: the name of the file to make executable
    """
    if os.path.exists(filename):
        st = os.stat(filename)
        mode = st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
        os.chmod(filename, mode)


def copyfile(source, destination, force=True):
    """copy a file from a source to its destination."""
    if os.path.exists(destination) and force is True:
        os.remove(destination)
    shutil.copyfile(source, destination)
    return destination


def write_file(filename, content, mode="w"):
    """write_file will open a file, "filename" and write content, "content"
    and properly close the file
    """
    with codecs.open(filename, mode, encoding="utf-8") as filey:
        filey.writelines(content)
    return filename


def write_json(json_obj, filename, mode="w", print_pretty=True):
    """write_json will (optionally,pretty print) a json object to file

    Parameters
    ==========
    json_obj: the dict to print to json
    filename: the output file to write to
    pretty_print: if True, will use nicer formatting
    """
    with codecs.open(filename, mode, encoding="utf-8") as filey:
        if print_pretty:
            filey.writelines(json.dumps(json_obj, indent=4, separators=(",", ": ")))
        else:
            filey.writelines(json.dumps(json_obj))
    return filename


def read_file(filename, mode="r", readlines=True):
    """write_file will open a file, "filename" and write content, "content"
    and properly close the file
    """
    with codecs.open(filename, mode, encoding="utf-8") as filey:
        if readlines is True:
            content = filey.readlines()
        else:
            content = filey.read()
    return content


def read_json(filename, mode="r"):
    """read_json reads in a json file and returns
    the data structure as dict.
    """
    with codecs.open(filename, mode, encoding="utf-8") as filey:
        data = json.load(filey)
    return data



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from __future__ import print_function

from scif.logger import bot
from subprocess import Popen, PIPE, STDOUT
import os
import shlex


################################################################################
# Software Versions
################################################################################


def get_installdir():
    """get_installdir returns the installation directory of the application"""
    return os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


def run_command(cmd, sudo=False, quiet=False):
    """run_command uses subprocess to send a command to the terminal.

    Parameters
    ==========
    cmd: the command to send, should be a list for subprocess
    sudo: if True (default False) command will be run asking for sudo
    quiet: skip printing output to the terminal
    """
    if not isinstance(cmd, list):
        cmd = shlex.split(cmd)

    if sudo is True:
        cmd = ["sudo"] + cmd

    process = Popen(cmd, stderr=STDOUT, stdout=PIPE)

    # Iterate through the output
    result = ""
    while True:
        output = process.stdout.readline()

        # Encode output to utf-8, if appropriate
        try:
            output = output.decode("utf-8")
        except:
            pass

        if output == "" and process.poll() is not None:
            break
        if output:
            result = "%s%s" % (result, output)
            if not quiet:
                print(output, end="")

    # Get the return code, assemble result
    output = {"message": result, "return_code": process.poll()}

    return output



#!/usr/bin/python

"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.utils import get_installdir
import unittest
import tempfile
import shutil
import json
import os

print("############################################################ test_utils")


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.pwd = get_installdir()
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_write_read_files(self):
        """test_write_read_files will test the functions write_file and read_file"""
        print("Testing utils.write_file...")
        from scif.utils import write_file

        tmpfile = tempfile.mkstemp()[1]
        os.remove(tmpfile)
        write_file(tmpfile, "hello!")
        self.assertTrue(os.path.exists(tmpfile))

        print("Testing utils.read_file...")
        from scif.utils import read_file

        content = read_file(tmpfile)[0]
        self.assertEqual("hello!", content)

        from scif.utils import write_json

        print("Testing utils.write_json...")
        print("...Case 1: Providing bad json")
        bad_json = {"Wakkawakkawakka'}": [{True}, "2", 3]}
        tmpfile = tempfile.mkstemp()[1]
        os.remove(tmpfile)
        with self.assertRaises(TypeError) as cm:
            write_json(bad_json, tmpfile)

        print("...Case 2: Providing good json")
        good_json = {"Wakkawakkawakka": [True, "2", 3]}
        tmpfile = tempfile.mkstemp()[1]
        os.remove(tmpfile)
        write_json(good_json, tmpfile)
        with open(tmpfile, "r") as filey:
            content = json.loads(filey.read())
        self.assertTrue(isinstance(content, dict))
        self.assertTrue("Wakkawakkawakka" in content)

    def test_run_command(self):
        """test running a command"""
        print("Testing utils.run_command")
        from scif.utils import run_command

        result = run_command("echo hello world")
        self.assertTrue("hello world" in result["message"])
        self.assertEqual(0, result["return_code"])

    def test_get_installdir(self):
        """get install directory should return the base of where singularity
        is installed
        """
        print("Testing utils.get_installdir")
        from scif.utils import get_installdir

        whereami = get_installdir()
        print(whereami)
        self.assertTrue("scif" in whereami)


if __name__ == "__main__":
    unittest.main()



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.defaults import SCIF_BASE
from scif.utils import which
import locale
import sys
import os


def _exec(self, app=None, interactive=False, exit=False):
    """exec is the underlying driver of both run and exec, taking a final
    SCIF and executing the command for the user.

    This function is called via self._exec, and the expectation is that
    self.run or self.exec has been called first to prepare the environment
    and SCIF settings.

    If a user wants to preserve an environment variable from the console
    it can be referenced with [e], for example $SCIF_DATA --> [e]SCIF_DATA

    Parameters
    ==========
    app: the name of the application to execute a command to
    interactive: if True, us os.system directly
    exit: exit with return code from command (for test)

    """

    name = ""
    if app is not None and app in self.apps():
        name = "[%s] " % app

    # If the entry folder still not set, we don't have an app
    if self._entry_folder is None:
        self._entry_folder = SCIF_BASE

    # Change directory to the relevant place
    os.chdir(self._entry_folder)

    # export environment
    runtime_environ = self.export_env()

    # Execv to the entrypoint
    executable = which(self._entry_point[0])

    # Exit if command not found
    args = ""
    if executable is None:
        bot.exit("%s not found." % self._entry_point[0])

    if len(self._entry_point) > 1:
        if exit is False:
            args = " ".join(self._entry_point[1:])
        else:
            args = self._entry_point[1:]

    # Are we executing a command as a string?
    if not isinstance(args, list):
        cmd = "%s %s" % (executable, args)
        bot.info("%sexecuting %s" % (name, cmd))

    # or a list with execv?
    else:
        bot.info("%sexecuting %s %s" % (name, executable, " ".join(args)))

    # Return output to console
    loc = locale.getdefaultlocale()[1]

    # A shell will run the command
    if interactive is True:
        # Will exit with error, if happens, otherwise exit with 0
        if exit is True:
            result = self._run_command(
                cmd=[executable] + args, spinner=False, quiet=self.quiet
            )
            sys.exit(result["return_code"])
        else:
            os.system("".join(cmd))

    else:
        for line in os.popen(cmd):
            try:
                print(line.rstrip())
            except:
                print(line.rstrip().encode(loc))


def execute(self, app, cmd=None, args=None):
    """execute a command in the context of an app. This means the following:

    1. Check that the app is valid for the client. Don't proceed otherwise
    2. Set the client app to be active
    3. update the environment to indicate the app is active
    4. set the entry point for exec to be relative to the app

    Parameters
    ==========
    app: the name of the scif app to execute a command to

    """
    self.activate(app, cmd, args)
    # checks for existence
    # sets _active to app's name
    # updates environment
    # sets entrypoint
    # sets entryfolder

    return self._exec(app)


def shell(self, app, cmd=None):
    """akin to execute, but specific for shell. In this case, we pass the
       calling process to the shell. We do the same steps as in run/exec:

    1. Check that the app is valid for the client. Don't proceed otherwise
    2. Set the client app to be active
    3. update the environment to indicate the app is active
    4. set the entry point for exec to be relative to the app

    Parameters
    ==========
    app: the name of the scif app to execute a command to

    """
    from scif.defaults import SCIF_SHELL

    if cmd is None:
        cmd = SCIF_SHELL

    self.activate(app, cmd)
    # checks for existence
    # sets _active to app's name
    # updates environment
    # sets shell entrypoint

    # interactive
    return self._exec(app, True)


def run(self, app=None, args=None):
    """run an app. This means the following:

    1. Check that the app is valid for the client. Don't proceed otherwise
    2. Set the client app to be active
    3. update the environment to indicate the app is active
    4. set the entry point and entry folder relevant to the app

    Parameters
    ==========
    app: the name of the scif app to run
    args: a list of one or more additional arguments to pass to runscript

    """
    interactive = False
    config = self.app(app)
    if "apprun" not in config:
        bot.debug("%s does not have a runscript." % app)
        interactive = True

    self.activate(app, args=args)  # checks for existence
    # sets _active to app's name
    # updates environment
    # sets entrypoint
    # sets entryfolder

    return self._exec(app, interactive=interactive)


def test(self, app=None, args=None):
    """test an app. This means the following:

    1. Check that the app is valid for the client. Don't proceed otherwise
    2. Set the client app to be active
    3. update the environment to indicate the app is active
    4. set the entry point and entry folder relevant to the app
    5. Run interactively, and return the entry code to the client

    Parameters
    ==========
    app: the name of the scif app to run. If none provided, all apps are tested.
    args: a list of one or more additional arguments to pass to runscript

    """

    # Does the application have a test script?

    if app in self.apps():
        self.activate(app, args=args)

        if not self._set_entrypoint(app, "SCIF_APPTEST", args):
            bot.info("No tests defined for this app.")
            sys.exit(1)

        return self._exec(app, interactive=True, exit=True)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.main.apps import app, apps, activate, deactivate, help, inspect, reset
from scif.main.base import ScifRecipe
from scif.main.commands import _exec, execute, run, shell, test
from scif.main.environment import (
    add_env,
    append_path,
    export_env,
    get_env,
    get_append_path,
    get_appenv,
    get_appenv_lookup,
    init_env,
    load_env,
    update_env,
)
from scif.main.helpers import run_command, set_entrypoint
from scif.main.preview import (
    preview,
    preview_apps,
    preview_base,
    init_app_preview,
    preview_runscript,
    preview_labels,
    preview_environment,
    preview_files,
    preview_commands,
    preview_recipe,
    preview_test,
)
from scif.main.setup import install_base, set_base, set_defaults
from scif.main.install import (
    init_app,
    install,
    install_apps,
    install_runscript,
    install_environment,
    install_help,
    install_script,
    install_labels,
    install_commands,
    install_files,
    install_recipe,
    install_test,
)


# We can eventually add logic here for customizing the client


# Commands
ScifRecipe.execute = execute
ScifRecipe._exec = _exec
ScifRecipe.run = run
ScifRecipe.test = test
ScifRecipe.shell = shell

# Helpers
ScifRecipe._run_command = run_command
ScifRecipe._set_entrypoint = set_entrypoint
ScifRecipe.help = help

# Environment
ScifRecipe.append_path = append_path
ScifRecipe._append_path = get_append_path
ScifRecipe._init_env = init_env
ScifRecipe.add_env = add_env
ScifRecipe.export_env = export_env
ScifRecipe.get_env = get_env
ScifRecipe.load_env = load_env
ScifRecipe.update_env = update_env


# Preview
ScifRecipe.preview = preview
ScifRecipe._preview_base = preview_base
ScifRecipe._preview_apps = preview_apps
ScifRecipe._init_app_preview = init_app_preview
ScifRecipe._preview_runscript = preview_runscript
ScifRecipe._preview_labels = preview_labels
ScifRecipe._preview_environment = preview_environment
ScifRecipe._preview_commands = preview_commands
ScifRecipe._preview_files = preview_files
ScifRecipe._preview_recipe = preview_recipe
ScifRecipe._preview_test = preview_test

# Setup
ScifRecipe._install_base = install_base
ScifRecipe.set_base = set_base
ScifRecipe.set_defaults = set_defaults

# Apps
ScifRecipe.get_appenv_lookup = get_appenv_lookup
ScifRecipe.get_appenv = get_appenv
ScifRecipe.app = app
ScifRecipe.apps = apps
ScifRecipe.activate = activate
ScifRecipe.deactivate = deactivate
ScifRecipe.inspect = inspect
ScifRecipe.reset = reset

# Installation
ScifRecipe.install = install
ScifRecipe._init_app = init_app
ScifRecipe._install_apps = install_apps
ScifRecipe._install_commands = install_commands
ScifRecipe._install_environment = install_environment
ScifRecipe._install_files = install_files
ScifRecipe._install_help = install_help
ScifRecipe._install_labels = install_labels
ScifRecipe._install_recipe = install_recipe
ScifRecipe._install_runscript = install_runscript
ScifRecipe._install_script = install_script
ScifRecipe._install_test = install_test



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import sys
import pwd
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    cmd = args.cmd

    if len(cmd) < 2:
        bot.warning("You must supply an appname and command to execute.")
        bot.custom(prefix="Example: ", message="scif exec app echo $SCIF_APPNAME")
        sys.exit(1)

    app = cmd.pop(0)

    # The next must be the program to execute
    command = cmd.pop(0)

    client = ScifRecipe(quiet=True, writable=args.writable)
    client.execute(app=app, cmd=command, args=cmd)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.utils import mkdir_p
from scif.main.helpers import parse_entrypoint
import os


def set_base(self, base="/", writable=True):
    """set the base (the root where to create /scif) and determine if
    it is writable

    Parameters
    ==========
    base: the full path to the root folder to create /scif
    """
    # The user is likely to give path to scif (should still work)
    base = base.strip("scif")

    if not os.path.exists(base):
        bot.exit("%s does not exist!" % base)

    base = "/%s" % os.path.abspath(base).strip("/")
    self._base = os.path.join(base, "scif")
    self.path_apps = "%s/apps" % self._base
    self.path_data = "%s/data" % self._base

    # Update the environment
    self.add_env("SCIF_DATA", self.path_data)
    self.add_env("SCIF_APPS", self.path_apps)

    # Check if it's writable
    if writable is True:
        if not os.access(base, os.W_OK):
            bot.exit("%s is not writable." % base)


def set_defaults(self, entry_point=None):
    """set the defaults for the SCIF state at start up.
    Usually this means parsing them from the environment. We hold these
    with the object so we can easily maintain and change state.
    """

    # The entrypoint is the runscript (or /bin/bash default)
    self._entry_point = parse_entrypoint(entry_point)

    # Set the default entry folder, changed to app if in context of app
    from scif.defaults import SCIF_ENTRYFOLDER

    self._entry_folder = SCIF_ENTRYFOLDER  # Defaults to None

    # The active app, coinciding with the entry_point
    self._active = None


def install_base(self):
    """create basic scif structure at the base for apps and metadata

    Parameters
    ==========
    base: the full path to the root folder to create /scif
    """
    if not hasattr(self, "_base"):
        bot.exit("Please set the base before installing to it.")

    bot.info("Installing base at %s" % self._base)

    mkdir_p(self.path_apps)
    mkdir_p(self.path_data)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

__version__ = "0.0.83"
AUTHOR = "Vanessa Sochat"
AUTHOR_EMAIL = "vsochat@stanford.edu"
NAME = "scif"
PACKAGE_URL = "http://www.github.com/vsoch/scif"
KEYWORDS = "the scientific filesystem"
DESCRIPTION = "a filesystem organization for scientific software and metadata"
LICENSE = "LICENSE"

INSTALL_REQUIRES = (
    ("six", {"min_version": "1.7.0"}),
    ("pygments", {"min_version": "2.1.3"}),
)

# Submodule Requirements

INSTALL_REQUIRES_ALL = INSTALL_REQUIRES



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from datetime import datetime
from scif.utils import mkdir_p, write_file, write_json
from scif.main.helpers import get_parts
import os


def preview(self, apps=None):
    """preview the complete setup for a scientific filesytem. This is useful
    to print out actions for install (without doing them).
    """
    self._preview_base()  # preview the folder structure
    self._preview_apps(apps)  # App install


def preview_base(self):
    """preview basic scif structure at the base for apps and metadata

    Parameters
    ==========
    base: the full path to the root folder to create /scif
    """
    if not hasattr(self, "_base"):
        bot.exit("Please set the base before preview.")

    bot.custom(prefix="[base] %s" % self._base, color="CYAN")
    bot.custom(prefix="[apps] %s" % self.path_apps, color="CYAN")
    bot.custom(prefix="[data] %s\n" % self.path_data, color="CYAN")


def preview_apps(self, apps=None):
    """install one or more apps to the base. If app is defined, only
    install app specified. Otherwise, install all found in config.
    """
    if apps in [None, []]:
        apps = self.apps()

    if not isinstance(apps, list):
        apps = [apps]

    for app in apps:
        # We must have the app defined in the config
        if app not in self._config["apps"]:
            bot.exit("Cannot find app %s in config." % app)

        # Make directories
        bot.newline()
        settings = self._init_app_preview(app)

        # Get the app configuration
        config = self.app(app)

        # Handle environment, runscript, labels
        self._preview_runscript(app, settings, config)
        self._preview_environment(app, settings, config)
        self._preview_labels(app, settings, config)
        self._preview_commands(app, settings, config)
        self._preview_files(app, settings, config)
        self._preview_recipe(app, settings, config)
        self._preview_test(app, settings, config)
        bot.newline()


def init_app_preview(self, app):
    """initialize an app, meaning adding the metadata folder, bin, and
    lib to it. The app is created at the base
    """
    settings = self.get_appenv_lookup(app)[app]

    for key in ["root", "lib", "bin", "data"]:
        val = settings["app%s" % key]
        bot.custom(prefix="[%s] %s" % (key, val), color="CYAN")

    return settings


def preview_runscript(self, app, settings, config):
    """preview the runscript for an app

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    if "apprun" in config:
        content = "\n".join(config["apprun"])
        bot.info("+ " + "apprun ".ljust(5) + app)
        print(settings["apprun"])
        print(settings["apphelp"])
        print(content)


def preview_test(self, app, settings, config):
    """preview the test for the app

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    if "apptest" in config:
        content = "\n".join(config["apptest"])
        bot.info("+ " + "apptest ".ljust(5) + app)
        print(settings["apptest"])
        print(content)


def preview_labels(self, app, settings, config):
    """preview labels for an app

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    lookup = ""
    if "applabels" in config:
        labels = config["applabels"]
        bot.info("+ " + "applabels ".ljust(5) + app)
        print(settings["applabels"])
        for line in labels:
            label, value = get_parts(line, default="")
            lookup += "%s=%s\n" % (label, value)
        print(lookup)
    return lookup


def preview_files(self, app, settings, config):
    """install files will add files (or directories) to a destination.
    If none specified, they are placed in the app base

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """

    if "appfiles" in config:
        files = config["appfiles"]
        bot.info("+ " + "appfiles ".ljust(5) + app)

        for pair in files:
            # Step 1: determine source and destination
            src, dest = get_parts(pair, default=settings["approot"])
            print("%s --> %s \n" % (src, dest))


def preview_commands(self, app, settings, config):
    """install will finally, issue commands to install the app.

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    cmd = ""
    if "appinstall" in config:
        cmd = config["appinstall"]
        bot.info("+ " + "appinstall ".ljust(5) + app)
        print("\n".join(cmd))
    return cmd


def preview_recipe(self, app, settings, config):
    """Write the initial recipe for the app to its metadata folder.

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    recipe_file = settings["apprecipe"]
    recipe = "# [scif] scientific filesystem recipe\n"
    recipe += "# [date] %s\n" % datetime.now().strftime("%b-%d-%Y")
    bot.info("+ " + "apprecipe ".ljust(5) + app)
    print(recipe_file)

    for section_name, section_content in config.items():
        content = "%s\n" % "\n".join(section_content)
        header = "%" + section_name + " %s" % app
        recipe += "%s\n%s\n" % (header, content)

    return recipe


def preview_environment(self, app, settings, config):
    """preview the environment section

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars

    """
    content = ""
    if "appenv" in config:
        content = "\n".join(config["appenv"])
        bot.info("+ " + "appenv ".ljust(5) + app)
        print(settings["appenv"])
        print(content)
    return content



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import sys
import pwd
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    apps = args.app
    client = ScifRecipe(quiet=True, writable=False)

    if len(apps) == 0:
        bot.info("Usage: scif help <hello-world>")
    for app in apps:
        client.help(app)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.client.utils import parse_input_preferences
from scif.logger import bot
import sys
import os


def main(args, parser, subparser):
    from scif.defaults import SCIF_SHELL as shell

    parsed = parse_input_preferences(args.recipe, quiet=args.quiet)

    recipe = parsed["recipe"]
    quiet = parsed["quiet"]
    app = parsed["app"]

    # The client will either load the recipe or the base

    lookup = {"ipython": ipython, "python": python, "bpython": bpython}

    shells = ["ipython", "python", "bpython"]

    # If the user asked for a specific shell via environment
    if shell is not None:
        shell = shell.lower()
        if shell in lookup:
            try:
                return lookup[shell](
                    recipe=recipe, app=app, quiet=quiet, writable=args.writable
                )
            except ImportError:
                pass

    # Otherwise present order of liklihood to have on system
    for shell in shells:
        try:
            return lookup[shell](
                recipe=recipe, app=app, quiet=quiet, writable=args.writable
            )
        except ImportError:
            pass


def ipython(recipe, app=None, quiet=False, writable=True):
    """embed the client with loaded recipe into an ipython session"""
    from scif.main import ScifRecipe
    from IPython import embed

    client = ScifRecipe(recipe, quiet=quiet, writable=writable)
    client.speak()
    if app is not None:
        client.activate(app)
    embed()


def bpython(recipe, app=None, quiet=False, writable=True):
    from scif.main import ScifRecipe
    import bpython

    client = ScifRecipe(recipe, quiet=quiet, writable=writable)
    client.speak()
    if app is not None:
        client.activate(app)
    bpython.embed(locals_={"client": client})


def python(recipe, app=None, quiet=False, writable=True):
    from scif.main import ScifRecipe
    import code

    client = ScifRecipe(recipe, quiet=quiet, writable=writable)
    client.speak()
    if app is not None:
        client.activate(app)
    code.interact(local={"client": client})



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.client.utils import parse_input_preferences
from scif.logger import bot
import sys
import json
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    # Inspect choices, r, e, l, a
    choices = [
        "r",
        "e",
        "l",
        "a",
        "f",
        "i",
        "h",
        "dump",
        "labels",
        "all",
        "files",
        "help",
        "install",
        "environment",
        "definition",
        "runscript",
    ]

    attributes = [x for x in args.attributes if x in choices]

    write_recipe = False
    if "dump" in attributes:
        write_recipe = True
        attributes.pop(attributes.index("dump"))

    apps = [x for x in args.attributes if x not in attributes]

    # First instantiate the client
    client = ScifRecipe(quiet=True, writable=False)

    # Filter down to apps that we want
    if len(apps) == 0:
        apps = client.apps()

    if len(attributes) == 0:
        attributes = ["a"]

    result = {}
    for app in apps:
        inspection = client.inspect(app, attributes)
        if len(inspection) > 0:
            result[app] = inspection

    if write_recipe is True:
        for app, atts in result.items():
            for key, val in atts.items():
                print("%" + key)
                print("\n".join(val) + "\n")
    else:
        print(json.dumps(result, indent=4, separators=(",", ": ")))



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.defaults import SCIF_SHELL, SCIF_BASE
import sys
import pwd
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    app = args.app
    client = ScifRecipe(quiet=True, writable=args.writable)

    # Only allow interactive shell if the base exists
    if os.path.exists(SCIF_BASE):
        client.shell(app, cmd=[SCIF_SHELL])



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.utils import run_command as run_cmd
from scif.defaults import SCIF_SHELL
from scif.logger import bot
import shlex
import os
import re


def get_parts(pair, default=None):
    """pair is expected to be a string with some key, value, and this function
    will replace any "=" with a space, and then parse the remainder
    for a first and second part (key and value pair).

    We likely will need to use regular expressions for pairs that have
    spaces or equals that need to be maintained.

    Parameters
    ==========
    pair should be a single string, likely with a key and value split by
         a space or equals sign.

    """
    pair = pair.replace("=", " ")
    parts = pair.split(" ")
    src = parts[0].strip()
    if len(parts) == 1:
        dest = default
    else:
        dest = parts[1].strip()
    return src, dest


def run_command(self, cmd, spinner=True, quiet=True):
    """run_command will run a command (a list) and wrap in a spinner. A
    result (dict) with message and return code is returned. If the
    return value is not 0, an error is issed and we exit
    """
    if spinner is True:
        bot.spinner.start()

    result = run_cmd(cmd, quiet=quiet)

    if spinner is True:
        bot.spinner.stop()

    retval = result["return_code"]

    # Beep boop, error!
    if retval != 0:
        bot.exit("Return code %s" % retval, retval)

    return result


def set_entrypoint(self, app, config_key="SCIF_APPRUN", args=None):
    """if the value is defined in the config and the file exists, set
    the entrypoint for the app to execute the script.

    Parameters
    ==========
    key: the entry in the config (a filename) to check for existence. If
         it exists, then set it's execution using the default shell to
         be the entrypoint.

    returns True if the config entry and file are found, False otherwise
    """

    if app in self.apps():
        config = self.get_appenv(app)

        if config_key in config:
            if os.path.exists(config[config_key]):
                self._entry_point = [SCIF_SHELL, config[config_key]]
                if args is not None:
                    args = parse_entrypoint(args)
                    self._entry_point += args
                return True
    return False


def parse_entrypoint(entry_point=None):
    """parse entrypoint will return a list, where the first argument is the
    executable, followed by arguments. This function also works to
    parse arguments into a list.

    Parameters
    ==========
    entry_point: the entry point for the application, is
    """

    if entry_point is None:
        from scif.defaults import SCIF_ENTRYPOINT as entry_point

    # We expect an [executable, arg1, arg2]
    if not isinstance(entry_point, list):
        entry_point = [entry_point]

    # Special characters in the entrypoint should be replaced
    #        [e] in the command or entrypoint: environment vars --> $
    #        [out] in the command or entrypoint: environment vars --> >
    #        [in] in the command or entrypoint: environment vars --> <
    #        [pipe] in the command or entrypoint: environment vars --> |

    entry_point = " ".join(entry_point)
    entry_point = re.sub("\[e\]", "$", entry_point)
    entry_point = re.sub("\[out\]", ">", entry_point)
    entry_point = re.sub("\[in\]", "<", entry_point)
    entry_point = re.sub("\[pipe\]", "|", entry_point)
    entry_point = re.sub("\[append\]", "|", entry_point)

    # Split into executable, arguments
    return shlex.split(entry_point)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.main.parser import load_filesystem, load_recipe
import os
import sys

bot.level = 3


class ScifRecipe:
    """Create and work with a scif recipe. Usage typically looks like:

    recipe = ScifRecipe('sregistry.scif')

    Parameters
    ==========
    path: the path to the scif recipe flie.

    """

    def __init__(self, path=None, app=None, writable=True, quiet=False):
        """initialize the scientific filesystem by creating a scif folder
        at the base, and loading the recipe to fill it.

        Parameters
        ==========
        path: is the first paramter, not required to initialize an empty
              client session. The logic proceeds as follows:
              1. If path is not defined, we want (empty) interactive session
              2. We derive the base from the environment SCIF_BASE
        """

        # 0. base determined from environment
        from scif.defaults import SCIF_BASE

        self.set_defaults()
        self.quiet = quiet

        # If recipe path not provided, try default base
        if path is None:
            path = SCIF_BASE

        # 1. Determine if path is a recipe or base
        if path is not None:
            self.set_base(SCIF_BASE, writable=writable)  # /scif
            self.load(path, app, quiet)  # recipe, environment

        # 2. Neither, development client
        else:
            bot.info("[skeleton] session!")
            bot.info('           load a recipe: client.load("recipe.scif")')
            bot.info('           change default base:  client.set_base("/")')

    def __str__(self):
        return "[scif]"

    def __repr__(self):
        return "[scif]"

    def speak(self):
        """the client should announce self given that the shell is being used."""
        if self._base is not None:
            apps = " | ".join(self.apps())
            bot.custom(prefix="%s %s" % (self, self._base), message=apps, color="CYAN")
        else:
            bot.info(self)

    def load(self, path, app=None, quiet=False):
        """load a scif recipe into the object

        Parameters
        ==========
        path: the complete path to the config (recipe file) to load, or
              root path of filesystem (that from calling function defaults to
              /scif)
        app:  if running with context of an active app, this will load the
              active app environment for it as well.
        """
        # 1. path is a recipe
        if os.path.isfile(path):
            self._config = load_recipe(path)

        # 2. path is a base
        elif os.path.isdir(path):
            self._config = load_filesystem(path, quiet=quiet)

        else:
            bot.warning("%s is not detected as a recipe or base." % path)
            self._config = None

        self.update_env(app)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
import sys
import pwd
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    apps = args.recipe

    if len(apps) > 0:
        recipe = apps.pop(0)

        if not os.path.exists(recipe):
            bot.exit("Cannot find recipe file %s" % recipe)

        client = ScifRecipe(recipe, writable=False)

        # Preview the entire recipe, or the apps chosen
        client.preview(apps)

    else:
        bot.info("You must provide a recipe file to preview!")



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""

from scif.logger import bot
from scif.utils import make_executable, mkdir_p, run_command, write_file, write_json
from scif.main.helpers import get_parts
import os


def install(self, app=None):
    """install recipes to a base. We assume this is the root of a system
    or container, and will write the /scif directory on top of it.
    If an app name is provided, install that app if it is found
    in the config. This function goes through all steps to:

    1. Install base folders to base, creating a folder for each app
    2. Install one or more apps to it, the config is already loaded
    """

    self._install_base()  # Generate the folder structure
    self._install_apps(app)  # App install


def init_app(self, app):
    """initialize an app, meaning adding the metadata folder, bin, and
    lib to it. The app is created at the base
    """
    settings = self.get_appenv_lookup(app)[app]

    # Create base directories for metadata
    for folder in ["appmeta", "appbin", "applib", "appdata"]:
        mkdir_p(settings[folder])
    return settings


def install_apps(self, apps=None):
    """install one or more apps to the base. If app is defined, only
    install app specified. Otherwise, install all found in config.
    """
    if apps in [None, ""]:
        apps = self.apps()

    if not isinstance(apps, list):
        apps = [apps]

    if len(apps) == 0:
        bot.warning("No apps to install. Load a recipe or base with .load()")

    for app in apps:
        # We must have the app defined in the config
        if app not in self._config["apps"]:
            bot.exit("Cannot find app %s in config." % app)

        # Make directories
        settings = self._init_app(app)

        # Get the app configuration
        config = self.app(app)

        # Get the app environment and export for install
        self.get_appenv(app, isolated=False, update=True)
        self.export_env(ps1=False)

        # Handle environment, runscript, labels
        self._install_runscript(app, settings, config)
        self._install_environment(app, settings, config)
        self._install_help(app, settings, config)
        self._install_labels(app, settings, config)
        self._install_files(app, settings, config)
        self._install_commands(app, settings, config)
        self._install_recipe(app, settings, config)
        self._install_test(app, settings, config)

        # After we install, in case interactive, deactivate last app
        self.deactivate(app)


def install_labels(self, app, settings, config):
    """install labels will add labels to the app labelfile

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    lookup = dict()
    if "applabels" in config:
        labels = config["applabels"]
        bot.level
        bot.info("+ " + "applabels ".ljust(5) + app)
        for line in labels:
            label, value = get_parts(line, default="")
            lookup[label] = value
        write_json(lookup, settings["applabels"])
    return lookup


def install_files(self, app, settings, config):
    """install files will add files (or directories) to a destination.
    If none specified, they are placed in the app base

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    if "appfiles" in config:
        files = config["appfiles"]
        bot.info("+ " + "appfiles ".ljust(5) + app)

        for pair in files:
            # Step 1: determine source and destination
            src, dest = get_parts(pair, default=settings["approot"])

            # Step 2: copy source to destination
            cmd = ["cp"]

            if os.path.isdir(src):
                cmd.append("-R")
            elif os.path.exists(src):
                cmd = cmd + [src, dest]
                result = self._run_command(cmd)
            else:
                bot.warning("%s does not exist, skipping." % src)


def install_commands(self, app, settings, config):
    """install will finally, issue commands to install the app.

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    if "appinstall" in config:
        # Change directory so the APP is $PWD
        pwd = os.getcwd()
        os.chdir(settings["approot"])

        # Set strict mode to ensure exit on error
        command = ["set -e"] + config["appinstall"]

        # issue install commands
        cmd = "\n".join(command)
        bot.info("+ " + "appinstall ".ljust(5) + app)
        retval = os.system(cmd)
        if retval != 0:
            bot.exit("Return value %s for install of %s" % (retval, app))

        # Go back to previous location
        os.chdir(pwd)


def install_recipe(self, app, settings, config):
    """Write the initial recipe for the app to its metadata folder.

    Parameters
    ==========
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)

    """
    recipe_file = settings["apprecipe"]
    recipe = ""

    for section_name, section_content in config.items():
        content = "\n".join(section_content)
        header = "%" + section_name
        recipe += "%s %s\n%s\n" % (header, app, content)

    write_file(recipe_file, recipe)
    return recipe


# Scripts


def install_script(self, section, app, settings, config, executable=False):
    """a general function used by install_runscript, install_help, and
    install_environment to write a script to a file from a config setting
    section

    Parameters
    ==========
    section: should be the name of the section in the config (e.g., apprun)
    app should be the name of the app, for lookup in config['apps']
    settings: the output of _init_app(), a dictionary of environment vars
    config: should be the config for the app obtained with self.app(app)
    executable: if the file is written, make it executable (defaults False)

    """
    if section in config:
        content = "\n".join(config[section])
        bot.info("+ " + section + " ".ljust(5) + app)
        write_file(settings[section], content)

        # Should we make the script executable (checks for exists)
        if executable is True:
            make_executable(settings[section])


def install_runscript(self, app, settings, config, executable=True):
    """install runscript will prepare the runscript for an app.
    the parameters are shared by _install_script
    """
    return self._install_script("apprun", app, settings, config, executable)


def install_environment(self, app, settings, config):
    """install will run the content to export environment variables, if defined
    the parameters are shared by _install_script
    """
    return self._install_script("appenv", app, settings, config)


def install_help(self, app, settings, config):
    """install will write the help section, if defined.
    the parameters are shared by _install_script
    """
    return self._install_script("apphelp", app, settings, config)


def install_test(self, app, settings, config, executable=True):
    """install test will prepare a test script for an app.
    the parameters are shared by _install_script
    """
    return self._install_script("apptest", app, settings, config, executable)



"""

Copyright (C) 2017-2024 Vanessa Sochat.

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed
with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

Modified from https://github.com/Visual-mov/Colorful-Julia (MIT License)

"""
from scif.client.utils import parse_input_preferences
from scif.logger import bot
import sys
import json
import os


def main(args, parser, subparser):
    from scif.main import ScifRecipe

    client = ScifRecipe(quiet=True, writable=False)
    apps = client.apps()

    for app in apps:
        inspection = client.inspect(app)
        if len(inspection) > 0:
            for key, val in inspection.items():
                print("%" + key)
                print("\n".join(val) + "\n")
